@@ -27,15 +27,16 @@
 
 void catalogs__initialize_cb                              (void);
 void catalogs__gth_browser_construct_cb                   (GthBrowser    *browser);
-void catalogs__gth_browser_update_sensitivity_cb          (GthBrowser    *browser);
+void catalogs__gth_browser_selection_changed_cb           (GthBrowser    *browser,
+							   int            n_selected);
 void catalogs__gth_browser_folder_tree_popup_before_cb    (GthBrowser    *browser,
 							   GthFileSource *file_source,
 							   GthFileData   *folder);
 GFile *      catalogs__command_line_files_cb              (GList         *files);
 GthCatalog * catalogs__gth_catalog_load_from_data_cb      (const void    *buffer);
+GthCatalog * catalogs__gth_catalog_new_for_uri_cb         (const char    *uri);
 void         catalogs__gth_browser_load_location_after_cb (GthBrowser    *browser,
-							   GthFileData   *location,
-					                   const GError  *error);
+							   GthFileData   *location);
 void         catalogs__gth_browser_update_extra_widget_cb (GthBrowser    *browser);
 void         catalogs__gth_browser_file_renamed_cb        (GthBrowser    *browser,
 							   GFile         *file,
