@@ -173,7 +173,8 @@ rotate (cairo_surface_t *image,
 	guchar           r0,
 	guchar           g0,
 	guchar           b0,
-	guchar           a0)
+	guchar           a0,
+	GthAsyncTask    *task)
 {
 	cairo_surface_t *image_with_background;
 	cairo_surface_t *rotated;
@@ -187,7 +188,7 @@ rotate (cairo_surface_t *image,
 	double           x2, y2;
 	int              x2min, y2min;
 	int              x2max, y2max;
-	double           fx, fy;
+	double           fx, fy, gx, gy;
 	guchar          *p_src, *p_new;
 	guchar          *p_src2, *p_new2;
 	guchar           r00, r01, r10, r11;
@@ -215,8 +216,8 @@ rotate (cairo_surface_t *image,
 		/* pre-multiply the background color */
 
 		image_with_background = _cairo_image_surface_copy (image);
-		p_src = cairo_image_surface_get_data (image);
-		p_new = cairo_image_surface_get_data (image_with_background);
+		p_src = _cairo_image_surface_flush_and_get_data (image);
+		p_new = _cairo_image_surface_flush_and_get_data (image_with_background);
 		src_rowstride = cairo_image_surface_get_stride (image);
 		new_rowstride = cairo_image_surface_get_stride (image_with_background);
 
@@ -247,8 +248,8 @@ rotate (cairo_surface_t *image,
 
 	rotated = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, new_width, new_height);
 
-	p_src = cairo_image_surface_get_data (image_with_background);
-	p_new = cairo_image_surface_get_data (rotated);
+	p_src = _cairo_image_surface_flush_and_get_data (image_with_background);
+	p_new = _cairo_image_surface_flush_and_get_data (rotated);
 	src_rowstride = cairo_image_surface_get_stride (image_with_background);
 	new_rowstride = cairo_image_surface_get_stride (rotated);
 
@@ -263,12 +264,10 @@ rotate (cairo_surface_t *image,
  *    |               |
  *    v10------------v11
  */
-#define INTERPOLATE(v, v00, v01, v10, v11, fx, fy) \
-	tmp = (1.0 - (fy)) * \
-	      ((1.0 - (fx)) * (v00) + (fx) * (v01)) \
-              + \
-              (fy) * \
-              ((1.0 - (fx)) * (v10) + (fx) * (v11)); \
+#define INTERPOLATE(v, v00, v01, v10, v11) \
+	tmp = gy * (gx * v00 + fx * v01) \
+	      + \
+	      fy * (gx * v10 + fx * v11); \
 	v = CLAMP (tmp, 0, 255);
 
 #define GET_VALUES(r, g, b, a, x, y) \
@@ -295,6 +294,18 @@ rotate (cairo_surface_t *image,
 
 	y = - half_new_height;
 	for (yi = 0; yi < new_height; yi++) {
+		if (task != NULL) {
+			gboolean cancelled;
+			double   progress;
+
+			gth_async_task_get_data (task, NULL, &cancelled, NULL);
+			if (cancelled)
+				goto out;
+
+			progress = (double) yi / new_height;
+			gth_async_task_set_data (task, NULL, NULL, &progress);
+		}
+
 		p_new2 = p_new;
 
 		x = - half_new_width;
@@ -305,8 +316,8 @@ rotate (cairo_surface_t *image,
 			if (high_quality) {
 				/* Bilinear interpolation. */
 
-				x2min = (int) x2;
-				y2min = (int) y2;
+				x2min = (int) floor (x2);
+				y2min = (int) floor (y2);
 				x2max = x2min + 1;
 				y2max = y2min + 1;
 
@@ -317,11 +328,13 @@ rotate (cairo_surface_t *image,
 
 				fx = x2 - x2min;
 				fy = y2 - y2min;
+				gx = 1.0 - fx;
+				gy = 1.0 - fy;
 
-				INTERPOLATE (r, r00, r01, r10, r11, fx, fy);
-				INTERPOLATE (g, g00, g01, g10, g11, fx, fy);
-				INTERPOLATE (b, b00, b01, b10, b11, fx, fy);
-				INTERPOLATE (a, a00, a01, a10, a11, fx, fy);
+				INTERPOLATE (r, r00, r01, r10, r11);
+				INTERPOLATE (g, g00, g01, g10, g11);
+				INTERPOLATE (b, b00, b01, b10, b11);
+				INTERPOLATE (a, a00, a01, a10, a11);
 
 				pixel = CAIRO_RGBA_TO_UINT32 (r, g, b, a);
 				memcpy (p_new2, &pixel, sizeof (guint32));
@@ -348,6 +361,8 @@ rotate (cairo_surface_t *image,
 		y += 1.0;
 	}
 
+	out:
+
 	cairo_surface_mark_dirty (rotated);
 	cairo_surface_destroy (image_with_background);
 
@@ -362,7 +377,8 @@ cairo_surface_t *
 _cairo_image_surface_rotate (cairo_surface_t *image,
 		    	     double           angle,
 		    	     gboolean         high_quality,
-		    	     cairo_color_t   *background_color)
+		    	     GdkRGBA         *background_color,
+		    	     GthAsyncTask    *task)
 {
 	cairo_surface_t *rotated;
 	cairo_surface_t *tmp = NULL;
@@ -380,10 +396,11 @@ _cairo_image_surface_rotate (cairo_surface_t *image,
 		rotated = rotate (image,
 				  -angle,
 				  high_quality,
-				  background_color->r * 255.0,
-				  background_color->g * 255.0,
-				  background_color->b * 255.0,
-				  background_color->a * 255.0);
+				  background_color->red * 255.0,
+				  background_color->green * 255.0,
+				  background_color->blue * 255.0,
+				  background_color->alpha * 255.0,
+				  task);
 	else
 		rotated = cairo_surface_reference (image);
 
