@@ -31,8 +31,8 @@ G_DEFINE_TYPE (GthFileToolRedo, gth_file_tool_redo, GTH_TYPE_FILE_TOOL)
 static void
 gth_file_tool_redo_update_sensitivity (GthFileTool *base)
 {
-	GtkWidget *window;
-	GtkWidget *viewer_page;
+	GtkWidget     *window;
+	GthViewerPage *viewer_page;
 
 	window = gth_file_tool_get_window (base);
 	viewer_page = gth_browser_get_viewer_page (GTH_BROWSER (window));
@@ -46,8 +46,8 @@ gth_file_tool_redo_update_sensitivity (GthFileTool *base)
 static void
 gth_file_tool_redo_activate (GthFileTool *base)
 {
-	GtkWidget *window;
-	GtkWidget *viewer_page;
+	GtkWidget     *window;
+	GthViewerPage *viewer_page;
 
 	window = gth_file_tool_get_window (base);
 	viewer_page = gth_browser_get_viewer_page (GTH_BROWSER (window));
@@ -72,11 +72,8 @@ gth_file_tool_redo_class_init (GthFileToolRedoClass *klass)
 static void
 gth_file_tool_redo_init (GthFileToolRedo *self)
 {
-	gboolean rtl;
-
-	rtl = gtk_widget_get_direction (GTK_WIDGET (self)) == GTK_TEXT_DIR_RTL;
-
-	gth_file_tool_construct (GTH_FILE_TOOL (self), rtl ? "edit-redo-rtl-symbolic" :
-							     "edit-redo-symbolic",
-						       _("Redo"), GTH_TOOLBOX_SECTION_FILE);
+	gth_file_tool_construct (GTH_FILE_TOOL (self),
+				 "edit-redo-symbolic",
+				 _("Redo"),
+				 GTH_TOOLBOX_SECTION_FILE);
 }
