@@ -57,6 +57,20 @@ delete_ready_cb (GError   *error,
 }
 
 
+static void
+delete_progress_cb (GObject    *object,
+		    const char *description,
+		    const char *details,
+		    gboolean    pulse,
+		    double      fraction,
+		    gpointer    user_data)
+{
+	GthDeleteTask *self = user_data;
+
+	gth_task_progress (GTH_TASK (self), description, details, pulse, fraction);
+}
+
+
 static void
 gth_delete_task_exec (GthTask *task)
 {
@@ -64,14 +78,13 @@ gth_delete_task_exec (GthTask *task)
 
 	self = GTH_DELETE_TASK (task);
 
-	gth_task_progress (task, _("Deleting files"), NULL, TRUE, 0.0);
-
-	_g_delete_files_async (self->priv->file_list,
-			       TRUE,
-			       TRUE,
-			       gth_task_get_cancellable (task),
-			       delete_ready_cb,
-			       self);
+	_g_file_list_delete_async (self->priv->file_list,
+				   TRUE,
+				   TRUE,
+				   gth_task_get_cancellable (task),
+				   delete_progress_cb,
+				   delete_ready_cb,
+				   self);
 }
 
 
